/* ALTERNATIVE APPROACHES  : 
 * 1. Multiple columns. (one implemented)
 * 2. Multiple rows using unions.
 * 3. Flag values as it is.
*/
/**
 * This system procedure is used to return the list  
 * of indexes defined in the current database.
 */
CREATE PROCEDURE psp_indexes
(   
	in :database_qual   VARCHAR(20) = DATABASE(), -- Name of the database
    in :table_name      VARCHAR(255) = NULL -- Name of the table
)

RETURNS
(
    TABLE_QUALIFIER     VARCHAR (20),  -- Name of the database that contains the table
    TABLE_OWNER         VARCHAR (20),  -- owner of the table
    TABLE_NAME          VARCHAR(255),  -- Name of the table
    INDEX_NAME          VARCHAR(255),  -- Name of the index
    INDEX_TYPE          CHAR(20), 	    -- Type of the index - Primary/Foreign 
    COLUMN_NAME         VARCHAR(255),  -- Name of the column
    ORDINAL_POSITION    USMALLINT,  -- Ordinal position of the index
    DUPLICATES_ALLOWED  CHAR(3), -- Duplicate Index
    UPDATABLE           CHAR(3), -- Modifiable Index
    CASE_SENSITIVE      CHAR(3), -- Case sensitive Index
    ASC_DESC		    CHAR(1), -- Asc or Desc index
    NAMED_INDEX		    CHAR(3)  -- Named or UnNamed Index
);

BEGIN
    -- Variable to hold the dynamic query
    declare :STMT_PSP_INDEXES LONGVARCHAR;
    declare :STMT_PSP_INDEXES2 LONGVARCHAR;
    declare :WithUnion INT;

    -- Check if the database qualifier specified is a blank string
	if (rtrim(:database_qual) = '') then
		SIGNAL 'S1000', 'Please enter a valid database name. Database name cannot be a blank string'; 
	end if;

     -- Check if the table_name specified is a blank string
	if (rtrim(:table_name) = '') then
		SIGNAL 'S1000', 'Table name cannot be a blank string'; 
	end if;

    -- If database qualifier is null, use current database
	SET :database_qual = IFNULL(:database_qual, DATABASE());

	/* If table name is not supplied or null value
	   match all */	
	SET :table_name = IFNULL(:table_name,'%');

	if ((SELECT COUNT(*) FROM X$File) = (SELECT COUNT(*) FROM X$File WHERE Xf$Name Like :table_name)) then
		SET :WithUnion = 0;
	else
		SET :WithUnion = 1;
	end if;

	/* Enumerate indexes for the given table */
	SET :stmt_psp_indexes2 = ' SELECT DISTINCT ''' + 
							:database_qual + /* TABLE_QUALIFIER */ 
	   						''', null, 	' +	 /* TABLE_OWNER */	
	   						' A.Xf$Name, ' + /* TABLE_NAME */
							' RTRIM(if(A.Xf$Flags & 16 = 16, ''key''+CAST(B.Xi$Number AS VARCHAR), C.Xe$Name)) Name, ' + /* INDEX_NAME*/
							' if(B.Xi$Flags & 16384 = 16384, ' +/* INDEX_TYPE */
							'''PRIMARY KEY INDEX''' +
							' , if(B.Xi$Flags & 8192 = 8192, ' +
							'''FOREIGN KEY INDEX''' +
							',' + '''NORMAL INDEX''' + ')),' +
							' D.Xe$Name, ' + /* COLUMN_NAME*/
							' B.Xi$Part, ' + /* ORDINAL_POSITION */
							'if(B.Xi$Flags & 1 = 1, ' + /* DUPLICATES_ALLOWED */
							'''YES''' + ' , ' + '''NO''' + ' ),' +
							' if(B.Xi$Flags & 2 = 2, ' +/* UPDATABLE */
							'''YES''' + ' , ' + '''NO''' + ' ),' +		
							' if(B.Xi$Flags & 32 = 32, ' + /* CASE_SENSITIVE */
							'''NO''' + ' , ' + '''YES''' + ' ),' +			
							' if(B.Xi$Flags & 64 = 64, ' + /* ASC_DESC */
							'''D''' + ' , ' + '''A''' + '),' +		
							' if(B.Xi$Flags & 128 = 128, ' + /* NAMED_INDEX */
							'''YES''' + ' , ' + '''NO''' + ' )' +	
							' FROM  "' + :database_qual + '".X$File A, "' +
										:database_qual + '".X$Index B, "' +
										:database_qual + '".X$Field C, "' +
										:database_qual + '".X$Field D ' +
							' WHERE  A.Xf$Name LIKE ''' + :table_name + 
							''' AND A.Xf$Id = B.Xi$File AND ' +
       						' A.Xf$Id = C.Xe$File AND ' +
							' ( B.Xi$Number = C.Xe$Offset OR A.Xf$Flags & 16 = 16 ) AND ' +
							' ( C.Xe$DataType = 227 OR  C.Xe$DataType = 255 OR A.Xf$Flags & 16 = 16 ) AND ' +
							' B.Xi$Field = D.Xe$Id ' +
							' ORDER BY A.Xf$Name, Name, B.Xi$Part ';

	SET :stmt_psp_indexes = :stmt_psp_indexes2;

	if (:WithUnion = 1) then

		SET :stmt_psp_indexes = ' SELECT DISTINCT ''' + 
							:database_qual + /* TABLE_QUALIFIER */ 
	   						''', null, 	' +	 /* TABLE_OWNER */	
	   						' A.Xf$Name, ' + /* TABLE_NAME */
	   						' RTRIM(E."INDEX_NAME") Name, ' +
							' if(B.Xi$Flags & 16384 = 16384, ' +/* INDEX_TYPE */
							'''PRIMARY KEY INDEX''' +
							' , if(B.Xi$Flags & 8192 = 8192, ' +
							'''FOREIGN KEY INDEX''' +
							',' + '''NORMAL INDEX''' + ')),' +
							' D.Xe$Name, ' + /* COLUMN_NAME*/
							' B.Xi$Part, ' + /* ORDINAL_POSITION */
							'if(B.Xi$Flags & 1 = 1, ' + /* DUPLICATES_ALLOWED */
							'''YES''' + ' , ' + '''NO''' + ' ),' +
							' if(B.Xi$Flags & 2 = 2, ' +/* UPDATABLE */
							'''YES''' + ' , ' + '''NO''' + ' ),' +		
							' if(B.Xi$Flags & 32 = 32, ' + /* CASE_SENSITIVE */
							'''NO''' + ' , ' + '''YES''' + ' ),' +			
							' if(B.Xi$Flags & 64 = 64, ' + /* ASC_DESC */
							'''D''' + ' , ' + '''A''' + '),' +		
							' if(B.Xi$Flags & 128 = 128, ' + /* NAMED_INDEX */
							'''YES''' + ' , ' + '''NO''' + ' )' +	
							' FROM  "' + :database_qual + '".X$File A, "' +
										:database_qual + '".X$Index B, "' +
										:database_qual + '".X$Field C, "' +
										:database_qual + '".X$Field D, ' +
								'( SELECT * FROM dbo.fSQLStatistics ( ''' + :database_qual + ''', ''' + :table_name + ''', 1 ) ) E ' +
							' WHERE  A.Xf$Name LIKE ''' + :table_name + 
							''' AND A.Xf$Id = B.Xi$File AND ' +
							' E."INDEX_NAME" IS NOT NULL AND ' +
							' RTRIM(A.Xf$Name) = E."TABLE_NAME" AND ' +
							' RTRIM(D.Xe$Name) = E."COLUMN_NAME" AND ' +
							' B.Xi$Part = E."SEQ_IN_INDEX" - 1 AND ' +
       						' A.Xf$Id = C.Xe$File AND ' +
							' ( B.Xi$Number = C.Xe$Offset OR A.Xf$Flags & 16 = 0 ) AND ' +
							' ( C.Xe$DataType = 227 OR  C.Xe$DataType = 255 OR A.Xf$Flags & 16 = 0 ) AND ' +
							' B.Xi$Field = D.Xe$Id AND ' +
							' B.Xi$Number IN ( SELECT DISTINCT Xi$Number FROM X$Index , ( SELECT DISTINCT Xf$ID FROM X$File ' +
							' WHERE  Xf$Name LIKE ''' + :table_name + 
							''' ) F WHERE Xi$File = F.Xf$ID AND ' +
							' Xi$Number NOT IN ( SELECT Xe$OffSet FROM X$Field WHERE Xe$DataType = 255 AND Xe$File = F.Xf$ID ) ) UNION '

		SET :stmt_psp_indexes = :stmt_psp_indexes + :stmt_psp_indexes2;

	end if;

	exec (:stmt_psp_indexes);

END; -- End of the stored procedure.

